/**************************************************************************
** File: validate.c
**
** Desc: Example that shows validating user password
**
**       This example shows how to correctly build an NLM that will have 
**       multiple threads verifying object passwords.  If you try to use
**       a single context, or a single connection, then requests will get 
**       mixed up.  This ends up causing good passwords to fail verification.
**
**
**   DISCLAIMER  
**  
**   Novell, Inc. makes no representations or warranties with respect to
**   any NetWare software, and specifically disclaims any express or
**   implied warranties of merchantability, title, or fitness for a
**   particular purpose.  
**
**   Distribution of any NetWare software is forbidden without the
**   express written consent of Novell, Inc.  Further, Novell reserves
**   the right to discontinue distribution of any NetWare software.
**   
**   Novell is not responsible for lost profits or revenue, loss of use
**   of the software, loss of data, costs of re-creating lost data, the
**   cost of any substitute equipment or program, or claims by any party
**   other than you.  Novell strongly recommends a backup be made before
**   any software is installed.   Technical support for this software
**   may be provided at the discretion of Novell.
**
**
** Programmers:
**
**    Ini   Who                  Firm
**    ---------------------------------------------------------------------
**    DWH   Dirk W. Howard       Novell Developer Support
**
**
** History:
**
**    When        Who      What
**    ---------------------------------------------------------------------
**    5-31-1996    DWH      First code.
**
*/

/**************************************************************************
** Function prototypes and macro definitions
*/

   /*------------------------------------------------
   ** Include files
   */
   #include <stdio.h>
   #include <signal.h>
   #include <process.h>
   #include <nwenvrn.h>
   #include <errno.h>
   #include <conio.h>
   #include <nwdsapi.h>
   #include <nwcntask.h>
   #include <string.h>


   /*------------------------------------------------
   ** Local function prototypes
   */
   void SignalHandler( int sig );
   void daughter(void *parm);


/**************************************************************************
** Global variable
*/
LONG     nlmResourceCnt = 0;
LONG     nlmExiting = FALSE;
int threadcnt;


/**************************************************************************
** main procedure
*/
void main() 
{ 
   int   x;

   /*------------------------------------------------
   ** Set counter for resource tracking
   */
   nlmResourceCnt++;
   threadcnt=0;

   /*------------------------------------------------
   ** register SIGTERM handler
   */
   signal( SIGTERM, SignalHandler );

   printf("Mainline is making threads..\n"); 
   for (x=0;x<10;x++) 
   {
      BeginThreadGroup(daughter,(void *) NULL, 8192, (void *)x);
      ThreadSwitchWithDelay();   /* Give each thread a chance to start */
   }

   /*------------------------------------------------
   ** wait for all daughter threads to die.. 
   */
   while ( (threadcnt > 0) && (nlmExiting == FALSE) )
      ThreadSwitchWithDelay();

   /*------------------------------------------------
   ** clean up 
   */
   printf("mainline is dead\n");
   nlmResourceCnt--;
   delay( 100 );
   return;
}


/**************************************************************************
** daughter procedure
*/
void daughter(void *parm)
{
   int               tnum = (int) parm;
   int               x;
   NWDSCCODE         err;
   NWDSContextHandle context;
   uint32            flags;
   int               success = 0;
   int               fail = 0;
   int               oldConnection;

   threadcnt++;
   nlmResourceCnt++;
   printf("starting thread %d..\n",tnum);

   /*------------------------------------------------
   ** Get connection and context for this thread.  This connection is not
   ** an NCP connection, so it does not count against the user license
   ** limit of the server.
   */
   oldConnection = SetCurrentConnection( -1 );
   context = NWDSCreateContext();     

   /*------------------------------------------------
   ** Set the context flag such that we can use typeless names and that
   ** the names are the full specification
   */
   if (context==(unsigned long)ERR_CONTEXT_CREATION) 
   {
      printf( "Could not create context in thread %d\n", tnum );
      exit(0);
   }
   err = NWDSGetContext( context, DCK_FLAGS, &flags );
   if ( err )
   {
      /*------------------------------------------------
      ** Error getting flags
      */
      printf( "Error getting flags in thread %d\n", tnum );
      printf("thread %d is dying..\n",tnum);
      NWDSFreeContext( context );
      threadcnt--;
      return;
   }
   else
   {
      flags |= DCV_TYPELESS_NAMES;
      flags &= ~DCV_CANONICALIZE_NAMES;
      err = NWDSSetContext( context, DCK_FLAGS, &flags );
      if ( err )
      {
         printf( "Error setting flags in thread %d\n", tnum );
         printf("thread %d is dying..\n",tnum);
         NWDSFreeContext( context );
         threadcnt--;
         return;
      }
   }

   /*------------------------------------------------
   ** Loop through the verification process 10 times
   */
   for (x=0;x<10;x++) 
   {
      /*------------------------------------------------
      ** If the NLM is exiting, then drop out of this loop
      */
      if ( nlmExiting )
         break;

      /*------------------------------------------------
      ** If this is not thread 1 then use the good password
      */
      if (tnum!=1) 
         err=NWDSVerifyObjectPassword( context, 0, 
            "User01.novell", "GOODPASS" );
      /*------------------------------------------------
      ** otherwise use the bad password
      */
      else 
         err=NWDSVerifyObjectPassword( context, 0,
            "User01.novell", "BADPASS" );

      /*------------------------------------------------
      ** tabulate results
      */
      if ( err == 0 )
         success++;
      else
         fail++;

      ThreadSwitchWithDelay();
   }

   /*------------------------------------------------
   ** Print results, cleanup and exit
   */
   printf("thread %d | Success: %02d  Failure: %02d\n", tnum, success, fail );
   NWDSFreeContext( context );
   ReturnConnection( SetCurrentConnection( oldConnection ) );
   threadcnt--;
   nlmResourceCnt--;
   return;
}


/**************************************************************************
** SignalHandler
*/
void SignalHandler( int sig )
{
   switch ( sig )
   {
      case SIGTERM:
         nlmExiting = TRUE;
         while ( nlmResourceCnt )
            ThreadSwitchWithDelay();
         break;
   }
   return;
}


